/* foXman
 * Copyright (C) 2000-2001 Josua Groeger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "FDAppMainWindow.h"

FXDEFMAP (FDAppMainWindow) FDAppMainWindowMap[]=
{
  FXMAPFUNC (SEL_COMMAND, FDAppMainWindow::ID_OPEN , FDAppMainWindow::onOpen),
  FXMAPFUNC (SEL_COMMAND, FDAppMainWindow::ID_ABOUT, FDAppMainWindow::onAbout),
  FXMAPFUNC (SEL_COMMAND, FDAppMainWindow::ID_HELP , FDAppMainWindow::onHelp),
};

FXIMPLEMENT (FDAppMainWindow, FXMainWindow, FDAppMainWindowMap, ARRAYNUMBER (FDAppMainWindowMap));

FDAppMainWindow::FDAppMainWindow (FDApp *a, const FXString &name, short section, char *manpage,
                                  FXIcon *ic= NULL, FXIcon *mi= NULL, FXuint opts= DECOR_ALL,
				  FXint x=0, FXint y=0, FXint w=0, FXint h=0):
                 FXMainWindow (a, name, ic, mi, opts, x,y,w,h, 0,0)
{
  // Menubar
  FXToolbarShell *menuShell= new FXToolbarShell (this);
  FXMenubar *menubar= new FXMenubar (this, menuShell, LAYOUT_FILL_X|FRAME_RAISED);
  new FXToolbarGrip (menubar, menubar, FXMenubar::ID_TOOLBARGRIP);
  
  // Toolbar
  FXToolbarShell *toolShell= new FXToolbarShell (this);
  FXToolbar *toolbar= new FXToolbar (this, toolShell, LAYOUT_FILL_X|FRAME_RAISED);
  new FXToolbarGrip (toolbar, toolbar, FXToolbar::ID_TOOLBARGRIP);
  
  // File menu
  fileMenu = new FXMenuPane (this);
  new FXMenuTitle (menubar, "&File", NULL, fileMenu);

  new FXMenuCommand (fileMenu, "&Open\tCtl-O", a->ico().insert ("mini/foXfolder_open.xpm"), this, ID_OPEN);
  new FXMenuSeparator (fileMenu);
  new FXMenuCommand (fileMenu, "&Quit\tCtl-Q", a->ico().insert ("mini/foXquit.xpm"),
                     a, FXApp::ID_QUIT, MENU_DEFAULT);

  // Help menu
  helpMenu = new FXMenuPane (this);
  new FXMenuTitle (menubar, "&Help", NULL, helpMenu, LAYOUT_RIGHT);
  
  new FXMenuCommand (helpMenu, "&Contents\tF1", a->ico().insert ("mini/foXhelp.xpm"), this, ID_HELP);
  new FXMenuSeparator (helpMenu);
  new FXMenuCommand (helpMenu, "&About foXman...", a->ico().insert ("mini/foXman.xpm"),
                     this, ID_ABOUT, MENU_DEFAULT);
  
  // Toolbar elements
  new FXButton (toolbar, "&Open", a->ico().insert ("mini/foXfolder_open.xpm"),
                this, ID_OPEN, ICON_BEFORE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED);
  
  manpageField= new FXTextField (toolbar, 15, NULL, 0, LAYOUT_RIGHT|FRAME_NORMAL);
  new FXLabel (toolbar, "Manpage:", a->ico().insert ("mini/foXdocument.xpm"),
               LAYOUT_RIGHT|ICON_BEFORE_TEXT|FRAME_THICK);
  
  new FXVerticalSeparator (toolbar, LAYOUT_FILL_Y|LAYOUT_RIGHT);
  
  sectionCombo= new FXComboBox (toolbar, 15, 11, NULL, 0, LAYOUT_RIGHT|COMBOBOX_STATIC|FRAME_NORMAL);
  new FXLabel (toolbar, "Section:", a->ico().insert ("mini/foXfolder.xpm"),
               LAYOUT_RIGHT|ICON_BEFORE_TEXT|FRAME_THICK);

  // Text
  FXPacker *textPacker= new FXPacker (this, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_NORMAL, 0,0,0,0, 0,0,0,0,0,0);
  text= new FXText (textPacker, NULL, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|TEXT_READONLY);
  
  // sectionCombo
  sectionCombo->appendItem ("(First matching manual page)");
  sectionCombo->appendItem ("1: Commands");
  sectionCombo->appendItem ("2: System calls");
  sectionCombo->appendItem ("3: Library calls");
  sectionCombo->appendItem ("4: Special files");
  sectionCombo->appendItem ("5: File formats and conventions");
  sectionCombo->appendItem ("6: Games");
  sectionCombo->appendItem ("7: Macro packages and conventions");
  sectionCombo->appendItem ("8: System management commands");
  sectionCombo->appendItem ("9: Kernel routines");
  sectionCombo->appendItem ("(All matching manual pages)");
  sectionCombo->setCurrentItem (0);

  openManpage (section, manpage);
}

FDAppMainWindow::~FDAppMainWindow ()
{
  delete fileMenu;
  delete helpMenu;
}

void FDAppMainWindow::openManpage (short section, char *manpage)
{
  FXString string;
  FXString cmd ("man ");
  
  if (section< 0 || section>= sectionCombo->getNumItems()) section= 0;
  if (section)
  {
    cmd+= (section==10)? "-a": FXStringVal (section);
    cmd+= ' ';
  }
  cmd+= manpage;
  cmd+= " 2>&1";
  
  FILE *file= popen (cmd.text (), "r"); 
  if (!file)
  {
    setTitle ("foXman");
    perror ("popen");
    sectionCombo->setCurrentItem (0);
    manpageField->setText ("");
  }
  else
  {
    sectionCombo->setCurrentItem (section);
    manpageField->setText (manpage);
    
    FXString title ("foXman: ");
    title+= cmd;
    title.trunc (title.size ()- 6); // remove " 2>&1"
    setTitle (title);
    
    // Get man's result (line by line)
    char line[1024], tmp_line[1024];
    while (fgets (line, 1024, file))
    {
      for (int i= 0; line[i]; i++)
      {
        // Remove the formatting characters
        if ((line[i]== '_' && line[i+1]== 0x08) || line[i]== 0x08)
        {
          strcpy (tmp_line, &line[i+2]);
          line[i]= 0;
          strcat (line, tmp_line);
        }
      }
      string.append (line);
    }
  }
  text->setText (string);
}

long FDAppMainWindow::onOpen (FXObject*, FXSelector, void*)
{
  openManpage (sectionCombo->getCurrentItem (), (char*) manpageField->getText ().text ());
  return 1;
}

long FDAppMainWindow::onAbout (FXObject*, FXSelector, void*)
{
  ((FDApp*) getApp ())->showAbout (this, "foXman", VERSION, "foXdesktop man page viewer", "foXman.xpm",
                                   "(C) 2000-2001 Josua Groeger <fractalus@gmx.net>", "http://foxdesktop.sourceforge.net");
  return 1;
}

long FDAppMainWindow::onHelp (FXObject*, FXSelector, void*)
{
  ((FDApp*) getApp ())->showHelp ("foXman");
  return 1;
}
